﻿// 京
#include "CInputWave.h"


#include <iostream>
#include <Windows.h>
#include <MMSystem.h>
//#include <tchar.h>

////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
////////////////////////////////////////////////////////////////////////////////
CInputWave::CInputWave()
{
	// Waveフォーマット
	m_WaveFormat.wFormatTag      = WAVE_FORMAT_PCM;
	m_WaveFormat.nChannels       = 1;
	//m_WaveFormat.nSamplesPerSec  = 22050;
	m_WaveFormat.nSamplesPerSec  = 20800;
	m_WaveFormat.wBitsPerSample  = 8;
	m_WaveFormat.nBlockAlign     = m_WaveFormat.wBitsPerSample / 8 * m_WaveFormat.nChannels;
	m_WaveFormat.nAvgBytesPerSec = m_WaveFormat.nSamplesPerSec * m_WaveFormat.nBlockAlign;


	// Waveデバイスハンドル
	m_hWaveIn = 0;

	// Waveヘッダー
	m_WaveHeaders.allocate(3);

	// Waveバッファ
	int a_RecorMilliSecond = 100;
	m_WaveBuffers.allocate(10, a_RecorMilliSecond, m_WaveFormat.nAvgBytesPerSec * a_RecorMilliSecond / 1000);

	m_pFinishWaveBuffer = NULL;

	m_pDataFullCallBackFunction = NULL;

	// デバイスをオープンしている
	m_isOpen = false;
	// 入力中
	m_isInput = false;
}

////////////////////////////////////////////////////////////////////////////////
// デストラクタ
////////////////////////////////////////////////////////////////////////////////
CInputWave::~CInputWave()
{
	close();
}

////////////////////////////////////////////////////////////////////////////////
// 入力デバイスの状態を知らせるコールバック関数
////////////////////////////////////////////////////////////////////////////////
void CALLBACK CInputWave::waveInProc(HWAVEIN in_hWaveIn, UINT in_Message, DWORD in_Instance, DWORD in_Param1, DWORD in_Param2)
{
	// waveInOpen時にin_Instanceにクラスのポインタを格納しておく

	CInputWave* a_pInputWave = reinterpret_cast<CInputWave*>(in_Instance);
	a_pInputWave->callbackWave(in_hWaveIn, in_Message, in_Instance, in_Param1, in_Param2);
}

void CInputWave::callbackWave(HWAVEIN in_hWaveIn, UINT in_Message, DWORD in_Instance, DWORD in_Param1, DWORD in_Param2)
{
	// メッセージの種類ごとの処理を行う
	switch(in_Message){
	// デバイスが waveInClose 関数でクローズされたときに送信されます。
	case WIM_CLOSE:
		std::cout << "WIM_CLOSE" << std::endl;
		break;

	// デバイスが、waveInAddBuffer 関数で送信されたデータブロックの処理を終了したときに送信されます。
	case WIM_DATA:
		std::cout << "WIM_DATA" << std::endl;
		// ヘッダー後処理
		m_pFinishWaveBuffer = m_WaveHeaders.getReadPointer()->lpData;
		waveInUnprepareHeader(m_hWaveIn, m_WaveHeaders.getReadPointer(), sizeof(WAVEHDR));

		// 外部関数を呼び出す
		if(m_pDataFullCallBackFunction != NULL){
			(*m_pDataFullCallBackFunction)(m_WaveHeaders.getReadPointer());
		}

		// 録音が終了したのでWaveヘッダーを解放する
		m_WaveHeaders.nextReadBuffer();

		// 入力フラグが立っている場合、次のバッファをセットする
		if(m_isInput){
			addWaveBuffer();
		}
		break;

	// デバイスが waveInOpen 関数でオープンされたときに送信されます。
	case WIM_OPEN:
		std::cout << "WIM_OPEN" << std::endl;
		break;
	default:
		throw "error: WAVE Call Back";
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// 初期化
////////////////////////////////////////////////////////////////////////////////
void CInputWave::initialize()
{
}

////////////////////////////////////////////////////////////////////////////////
// デバイスを開く
////////////////////////////////////////////////////////////////////////////////
bool CInputWave::open()
{
	// open a wave device handle
	// ※開くデバイスも引数で指定できるようにする
	if(waveInOpen(&m_hWaveIn, WAVE_MAPPER, &m_WaveFormat, reinterpret_cast<DWORD>(waveInProc), reinterpret_cast<DWORD>(this), CALLBACK_FUNCTION) != MMSYSERR_NOERROR) {
		throw "failer open wave device handle";
		return false;
	}

	m_isOpen = true;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// デバイスを閉じる
////////////////////////////////////////////////////////////////////////////////
bool CInputWave::close()
{
	stop();

	if(m_isOpen){
		m_isOpen = false;
		// close a wave device handle
		waveInClose(m_hWaveIn);
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// add wave buffer in queue
////////////////////////////////////////////////////////////////////////////////
bool CInputWave::addWaveBuffer()
{
	while(!m_WaveHeaders.full() && !m_WaveBuffers.full()){
		// ヘッダーと格納用データのポインタを取得する
		WAVEHDR* a_pWaveHeader = m_WaveHeaders.getWritePointer();
		StructWaveBuffer* a_pWaveBuffer = m_WaveBuffers.getWritePointer();
		// 初期値をセットする
		a_pWaveHeader->lpData         = a_pWaveBuffer->m_pWaveBuffer;
		a_pWaveHeader->dwBufferLength = a_pWaveBuffer->m_BufferSize;
		a_pWaveHeader->dwFlags        = 0;
		// ポインタを次に進める
		m_WaveHeaders.nextWriteBuffer();
		m_WaveBuffers.nextWriteBuffer();

		// initialize a wave header
		// must use this function for initialize, because not open wave header structure
		switch(waveInPrepareHeader(m_hWaveIn, a_pWaveHeader, sizeof(WAVEHDR))){
		// 成功
		case MMSYSERR_NOERROR:
			break;

		// 別のスレッドがハンドルを使用中である
		case MMSYSERR_HANDLEBUSY:
			throw "waveInPrepareHeader: 別のスレッドがハンドルを使用中である ";
			break;
		// デバイスハンドルが無効である
		case MMSYSERR_INVALHANDLE:
			throw "waveInPrepareHeader: デバイスハンドルが無効である";
			break;
		// デバイスドライバがない
		case MMSYSERR_NODRIVER:
			throw "waveInPrepareHeader:デバイスドライバがない";
			break;
		// メモリを割り当てられない、またはロックできない
		case MMSYSERR_NOMEM:
			throw "waveInPrepareHeader: メモリを割り当てられない、またはロックできない";
			break;
		}

		// take a buffer for wave
		switch(waveInAddBuffer(m_hWaveIn, a_pWaveHeader, sizeof(WAVEHDR))){
		// 成功
		case MMSYSERR_NOERROR:
			break;

		// 別のスレッドがハンドルを使用中である
		case MMSYSERR_HANDLEBUSY:
			throw "waveInAddBuffer: 別のスレッドがハンドルを使用中である";
			break;

		// デバイスハンドルが無効である
		case MMSYSERR_INVALHANDLE:
			throw "waveInAddBuffer: デバイスハンドルが無効である";
			break;

		// デバイスドライバがない
		case MMSYSERR_NODRIVER:
			throw "waveInAddBuffer: デバイスドライバがない";
			break;

		// メモリを割り当てられない、またはロックできない
		case MMSYSERR_NOMEM:
			throw "waveInAddBuffer: メモリを割り当てられない、またはロックできない";
			break;

		// バッファが準備されていない
		case WAVERR_UNPREPARED:
			throw "waveInAddBuffer: バッファが準備されていない";
			break;
		}
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 入力（録音）を開始する
////////////////////////////////////////////////////////////////////////////////
bool CInputWave::start()
{
	// デバイスを開いていないならfalseを返す
	if(!m_isOpen){
		return false;
	}

	// バッファに残りがある場合、バッファを加える
	if(!m_WaveHeaders.full() && !m_WaveBuffers.full()){
		addWaveBuffer();
	}
	else{
		return true;
	}

	// waveInStartは実行中の場合、何度呼び出しても無視されるはずだが、
	// なぜかフリーズするため、バッファの残りを確認し呼び出すか判定する

	switch(waveInStart(m_hWaveIn)){
	// 成功
	case MMSYSERR_NOERROR:
		break;

	// 指定されたデバイスハンドルは無効です。
	case MMSYSERR_INVALHANDLE:
		throw "指定されたデバイスハンドルは無効です。";
		return false;

	// デバイスドライバが存在しません。
	case MMSYSERR_NODRIVER:
		throw "デバイスドライバが存在しません。";
		return false;

	// メモリを割り当てられないか、またはロックできません。
	case MMSYSERR_NOMEM:
		throw "メモリを割り当てられないか、またはロックできません。";
		return false;

	// その他の処理
	default:
		break;
	}
	m_isInput = true;
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 入力（録音）を停止する
////////////////////////////////////////////////////////////////////////////////
bool CInputWave::stop()
{
	// 録音中でないならfalseを返す
	if(!m_isInput){
		return false;
	}

	m_isInput = false;
	waveInStop(m_hWaveIn);


	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 音声データを取得する
////////////////////////////////////////////////////////////////////////////////
StructWaveBuffer* CInputWave::getWaveDataPointer()
{
	if(m_pFinishWaveBuffer == NULL){
		return NULL;
	}

	if(m_WaveBuffers.getReadPointer()->m_pWaveBuffer != m_pFinishWaveBuffer){
		StructWaveBuffer* temp = m_WaveBuffers.getReadPointer();
		m_WaveBuffers.nextReadBuffer();
		return temp;
	}
	return NULL;
}
