﻿// 京
#ifndef INC_CInputWave
#define INC_CInputWave

// インクルードは必ず必要な物だけ
// 内部で利用するだけの物はソースに記述

// 関数名はローワキャメルケースとする


#include "CWaveData.h"
#include "CRingBuffer.h"

// 音声入力完了後に呼び出す関数へのポインタ
// 今後もこの機能を残す場合、関数オブジェクトに置き換える
typedef void (*DataFullCallBack)(WAVEHDR*);

////////////////////////////////////////////////////////////////////////////////
// CInputWave
////////////////////////////////////////////////////////////////////////////////
class CInputWave
{
	////////////////////////////////////////////////////////////////////////////////
	// メンバ変数
	////////////////////////////////////////////////////////////////////////////////
private:
	// wave format
	WAVEFORMATEX m_WaveFormat;
	// Waveデバイスハンドル
	HWAVEIN m_hWaveIn;
	// Waveヘッダー
	typedef CRingBuffer<WAVEHDR> WaveHeaderTable;
	WaveHeaderTable m_WaveHeaders;
	// Waveバッファ
	//typedef CRingBuffer<StructWaveBuffer> WaveBufferTable;
	CWaveData m_WaveBuffers;
	char* m_pFinishWaveBuffer;

	// 関数ポインタ
	DataFullCallBack m_pDataFullCallBackFunction;

	// デバイスをオープンしている
	bool m_isOpen;
	// 録音中
	bool m_isInput;

	////////////////////////////////////////////////////////////////////////////////
	// コンストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	CInputWave();

	////////////////////////////////////////////////////////////////////////////////
	// デストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	virtual ~CInputWave();

	////////////////////////////////////////////////////////////////////////////////
	// static関数、コールバック関数
	////////////////////////////////////////////////////////////////////////////////
	// 入力デバイスの状態を知らせるコールバック関数
	static void CALLBACK waveInProc(HWAVEIN in_hWaveIn, UINT in_Message, DWORD in_Instance, DWORD in_Param1, DWORD in_Param2);
	void callbackWave(HWAVEIN in_hWaveIn, UINT in_Message, DWORD in_Instance, DWORD in_Param1, DWORD in_Param2);


	////////////////////////////////////////////////////////////////////////////////
	// 操作メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	// 初期化
	void initialize();
	// デバイスを開く
	bool open();
	// デバイスを閉じる
	bool close();
protected:
	// add wave buffer in queue
	bool addWaveBuffer();
public:
	// 入力（録音）を開始する
	bool start();
	// 入力（録音）を停止する
	bool stop();
	// 音声データを取得する
	StructWaveBuffer* getWaveDataPointer();

	////////////////////////////////////////////////////////////////////////////////
	// 属性メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	void setDataFullCallBack(DataFullCallBack in_pDataFullCallBack)
	{
		m_pDataFullCallBackFunction = in_pDataFullCallBack;
	}

	////////////////////////////////////////////////////////////////////////////////
	// イテレーション
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 状態
	////////////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////////////
	// 演算子
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 純粋仮想関数、定義のみ（非実装提供）
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 内部クラス
	////////////////////////////////////////////////////////////////////////////////


};

#endif //INC_CInputWave
