﻿// 京
#include "COutputWave.h"


#include <iostream>
#include <Windows.h>
#include <MMSystem.h>
//#include <tchar.h>


////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
////////////////////////////////////////////////////////////////////////////////
COutputWave::COutputWave()
{
	// Waveフォーマット
	m_WaveFormat.wFormatTag      = WAVE_FORMAT_PCM;
	m_WaveFormat.nChannels       = 1;
	//m_WaveFormat.nSamplesPerSec  = 22050;
	m_WaveFormat.nSamplesPerSec  = 20800;
	m_WaveFormat.wBitsPerSample  = 8;
	m_WaveFormat.nBlockAlign     = m_WaveFormat.wBitsPerSample / 8 * m_WaveFormat.nChannels;
	m_WaveFormat.nAvgBytesPerSec = m_WaveFormat.nSamplesPerSec * m_WaveFormat.nBlockAlign;


	// Waveデバイスハンドル
	m_hWaveOut = 0;

	// Waveヘッダー
	m_WaveHeaders.allocate(3);

	// Waveバッファ
	int a_RecorMilliSecond = 100;
	m_WaveBuffers.allocate(10, a_RecorMilliSecond, m_WaveFormat.nAvgBytesPerSec * a_RecorMilliSecond / 1000);

	//m_pFinishWaveBuffer = NULL;

	// デバイスをオープンしている
	m_isOpen = false;
	// 再生中
	m_isOutput = false;
}

////////////////////////////////////////////////////////////////////////////////
// デストラクタ
////////////////////////////////////////////////////////////////////////////////
COutputWave::~COutputWave()
{
	close();
}


////////////////////////////////////////////////////////////////////////////////
// 出力デバイスの状態を知らせるコールバック関数
////////////////////////////////////////////////////////////////////////////////
void CALLBACK COutputWave::waveOutProc(HWAVEOUT in_hWaveOut, UINT in_Message, DWORD in_Instance, DWORD in_Param1, DWORD in_Param2)
{
	// waveInOpen時にin_Instanceにクラスのポインタを格納しておく

	COutputWave* a_pOutputWave = reinterpret_cast<COutputWave*>(in_Instance);
	a_pOutputWave->callbackWave(in_hWaveOut, in_Message, in_Instance, in_Param1, in_Param2);
}

void COutputWave::callbackWave(HWAVEOUT in_hWaveOut, UINT in_Message, DWORD in_Instance, DWORD in_Param1, DWORD in_Param2)
{
	// メッセージの種類ごとの処理を行う
	switch(in_Message){
	// デバイスが waveOutClose 関数でクローズされたときに送信されます。
	case WOM_CLOSE:
		std::cout << "WOM_CLOSE" << std::endl;
		break;

	// デバイスドライバが、waveOutWrite 関数で送信されたデータブロックの処理を終了したときに送信されます。
	case WOM_DONE:
		std::cout << "WOM_DATA" << std::endl;
		// ヘッダー後処理
		//m_pFinishWaveBuffer = m_WaveHeaders.getReadPointer()->lpData;
		waveOutUnprepareHeader(m_hWaveOut, m_WaveHeaders.getReadPointer(), sizeof(WAVEHDR));

		// 再生が終了したのでWaveヘッダーを解放する
		m_WaveHeaders.nextReadBuffer();

		if(m_isOutput){
			addWaveBuffer();
		}
		break;

	// デバイスが waveOutOpen 関数でオープンされたときに送信されます。
	case WOM_OPEN:
		std::cout << "WOM_OPEN" << std::endl;
		break;
	default:
		throw "error: WAVE Call Back";
		break;
	}
}


////////////////////////////////////////////////////////////////////////////////
// 初期化
////////////////////////////////////////////////////////////////////////////////
void COutputWave::initialize()
{
}

////////////////////////////////////////////////////////////////////////////////
// デバイスを開く
////////////////////////////////////////////////////////////////////////////////
bool COutputWave::open()
{
	// デバイスを開く
	// 開くデバイスも引数で指定できるようにする
	if(waveOutOpen(&m_hWaveOut, WAVE_MAPPER, &m_WaveFormat, reinterpret_cast<DWORD>(waveOutProc), reinterpret_cast<DWORD>(this), CALLBACK_FUNCTION) != MMSYSERR_NOERROR) {
		//MessageBox(NULL, TEXT("WAVEデバイスのオープンに失敗しました。"), NULL, MB_ICONWARNING);
		throw "WAVEデバイスのオープンに失敗しました。";
		return false;
	}

	m_isOpen = true;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// デバイスを閉じる
////////////////////////////////////////////////////////////////////////////////
bool COutputWave::close()
{
	if(m_isOpen){
		m_isOpen = false;
		// デバイスクローズ関数
		waveOutClose(m_hWaveOut);
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// add wave buffer in queue
////////////////////////////////////////////////////////////////////////////////
bool COutputWave::addWaveBuffer()
{
	while(!m_WaveHeaders.full() && !m_WaveBuffers.empty()){
		// ヘッダーと格納用データのポインタを取得する
		WAVEHDR* a_pWaveHeader = m_WaveHeaders.getWritePointer();
		StructWaveBuffer* a_pWaveBuffer = m_WaveBuffers.getReadPointer();
		// 初期値をセットする
		a_pWaveHeader->lpData         = a_pWaveBuffer->m_pWaveBuffer;
		a_pWaveHeader->dwBufferLength = a_pWaveBuffer->m_BufferSize;
		a_pWaveHeader->dwFlags        = 0;
		// ポインタを次に進める
		m_WaveHeaders.nextWriteBuffer();
		m_WaveBuffers.nextReadBuffer();

		if(a_pWaveHeader->lpData == NULL){
			return false;
		}

		// ヘッダーを準備する
		waveOutPrepareHeader(m_hWaveOut, a_pWaveHeader, sizeof(WAVEHDR));

		// 出力（再生）を開始する
		waveOutWrite(m_hWaveOut, a_pWaveHeader, sizeof(WAVEHDR));
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 出力（再生）を開始する
////////////////////////////////////////////////////////////////////////////////
bool COutputWave::start()
{
	// デバイスを開いていないならfalseを返す
	if(!m_isOpen){
		return false;
	}

	addWaveBuffer();

	m_isOutput = true;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 出力（再生）を停止する
////////////////////////////////////////////////////////////////////////////////
bool COutputWave::stop()
{
	// 再生中でない場合falseを返す
	if(!m_isOutput){
		return false;
	}

	waveOutReset(m_hWaveOut);
	m_isOutput = false;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 音声データをセットする
////////////////////////////////////////////////////////////////////////////////
bool COutputWave::setWaveDataPointer(const StructWaveBuffer* in_pWaveBuffer)
{
	if(in_pWaveBuffer == NULL){
		return false;
	}

	StructWaveBuffer* temp = m_WaveBuffers.getWritePointer();

	if(temp == NULL){
		return false;
	}

	temp->m_BufferSize = in_pWaveBuffer->m_BufferSize;
	temp->m_RecorMilliSecond = in_pWaveBuffer->m_RecorMilliSecond;
	for(unsigned int i = 0; i < in_pWaveBuffer->m_BufferSize; i++){
		temp->m_pWaveBuffer[i] = in_pWaveBuffer->m_pWaveBuffer[i];
	}

	m_WaveBuffers.nextWriteBuffer();
	return true;
}
