﻿// 京
#ifndef INC_CRingBuffer
#define INC_CRingBuffer


#include <vector>

////////////////////////////////////////////////////////////////////////////////
// CRingBuffer
////////////////////////////////////////////////////////////////////////////////
template<typename T>
class CRingBuffer
{
	////////////////////////////////////////////////////////////////////////////////
	// メンバ変数
	////////////////////////////////////////////////////////////////////////////////
private:
	std::vector<T> m_Buffer;
	int m_ReadPosition;
	int m_WritePosition;

	////////////////////////////////////////////////////////////////////////////////
	// コンストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	CRingBuffer()
	{
		m_ReadPosition = 0;
		m_WritePosition = 0;
	}

	////////////////////////////////////////////////////////////////////////////////
	// デストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	virtual ~CRingBuffer()
	{
		clear();
	}

	////////////////////////////////////////////////////////////////////////////////
	// static関数、コールバック関数
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 操作メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	bool allocate(unsigned int in_Size)
	{
		if(in_Size < 2){
			return false;
		}

		m_Buffer.resize(in_Size);
		m_ReadPosition = 0;
		m_WritePosition = 0;
		return true;
	}

	void clear()
	{
		m_Buffer.clear();
		m_ReadPosition = 0;
		m_WritePosition = 0;
	}

	bool nextReadBuffer()
	{
		if(empty()){
			return false;
		}

		m_ReadPosition = (m_ReadPosition + 1) % m_Buffer.size();
		return true;
	}

	bool nextWriteBuffer()
	{
		if(full()){
			return false;
		}

		m_WritePosition = (m_WritePosition + 1) % m_Buffer.size();
		return true;
	}

	////////////////////////////////////////////////////////////////////////////////
	// 属性メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	size_t size() const
	{
		return m_Buffer.size();
	}

	T* at(size_t in_Index)
	{
		return &m_Buffer[in_Index];
	}

	const T* at(size_t in_Index) const
	{
		return &m_Buffer[in_Index];
	}

	T* getReadPointer()
	{
		if(empty()){
			return NULL;
		}
		return &m_Buffer[m_ReadPosition];
	}

	const T* getReadPointer() const
	{
		return &m_Buffer[m_ReadPosition];
	}

	T* getWritePointer()
	{
		return &m_Buffer[m_WritePosition];
	}

	const T* getWritePointer() const
	{
		return &m_Buffer[m_WritePosition];
	}

	////////////////////////////////////////////////////////////////////////////////
	// イテレーション
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 状態
	////////////////////////////////////////////////////////////////////////////////
public:
	// check if the buffer is full
	bool full() const
	{
		return m_ReadPosition == (m_WritePosition + 1) % m_Buffer.size();
	}

	// check if the buffer is empty
	bool empty() const
	{
		return m_ReadPosition == m_WritePosition;;
	}

	////////////////////////////////////////////////////////////////////////////////
	// 演算子
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 純粋仮想関数、定義のみ（非実装提供）
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 内部クラス
	////////////////////////////////////////////////////////////////////////////////


};

#endif //INC_CRingBuffer
