﻿#include <winsock2.h>
#include <iostream>

#include "CServerSocket.h"

////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
////////////////////////////////////////////////////////////////////////////////
CServerSocket::CServerSocket()
{
	m_PortNumber = 12345;
}

////////////////////////////////////////////////////////////////////////////////
// デストラクタ
////////////////////////////////////////////////////////////////////////////////
CServerSocket::~CServerSocket()
{
	uninitialize();
}

////////////////////////////////////////////////////////////////////////////////
// 初期化
////////////////////////////////////////////////////////////////////////////////
bool CServerSocket::initialize()
{
	// どっかでシングルトンを使って初期化しておく
	int a_Error = WSAStartup(MAKEWORD(2,0), &m_WsaData);
	if(a_Error != 0){
		switch(a_Error){
		case WSASYSNOTREADY:
			throw "WSASYSNOTREADY";
			break;
		case WSAVERNOTSUPPORTED:
			throw "WSAVERNOTSUPPORTED";
			break;
		case WSAEINPROGRESS:
			throw "WSAEINPROGRESS";
			break;
		case WSAEPROCLIM:
			throw "WSAEPROCLIM";
			break;
		case WSAEFAULT:
			throw "WSAEFAULT";
			break;
		}
		return false;
	}

	/*
	std::cout << "バージョン = " << static_cast<int>(wsaData.wHighVersion) << "." << static_cast<int>(wsaData.wHighVersion >> 8) << "\n"
	          << "description = " << wsaData.szDescription << "\n"
	          << "status = " << wsaData.szSystemStatus << "\n" << std::endl;
	*/

	m_ServerSocket = socket(AF_INET, SOCK_STREAM, 0);
	if(m_ServerSocket == INVALID_SOCKET){
		//std::cout << "socket failed: " << WSAGetLastError() << std::endl;
		return false;
	}

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 非初期化
////////////////////////////////////////////////////////////////////////////////
bool CServerSocket::uninitialize()
{
	closePort();
	WSACleanup();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// open port
////////////////////////////////////////////////////////////////////////////////
bool CServerSocket::openPort()
{
	// ソケットの設定
	m_ServerSocketAddress.sin_family = AF_INET;
	m_ServerSocketAddress.sin_port = htons(m_PortNumber);
	m_ServerSocketAddress.sin_addr.S_un.S_addr = INADDR_ANY;
	BOOL yes = 1;
	setsockopt(m_ServerSocket, SOL_SOCKET, SO_REUSEADDR, (const char *)&yes, sizeof(yes));
	bind(m_ServerSocket, reinterpret_cast<sockaddr*>(&m_ServerSocketAddress), sizeof(m_ServerSocketAddress));

	// TCPクライアントからの接続要求を待てる状態にする
	listen(m_ServerSocket, 5);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// close port
////////////////////////////////////////////////////////////////////////////////
bool CServerSocket::closePort()
{
	closesocket(m_ServerSocket);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// accept
////////////////////////////////////////////////////////////////////////////////
bool CServerSocket::accept(CSocket* io_pSocket)
{
	try{
		m_NonConnectedSockets.push_back(io_pSocket);
	}
	catch(...){
		return false;
	}
	if(!m_isExist){
		start(true);
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// クライアントからの接続を待つ
////////////////////////////////////////////////////////////////////////////////
void CServerSocket::run()
{
	// TCPクライアントからの接続要求を受け付ける
	if(!m_NonConnectedSockets.empty()){
		SOCKET a_ClientSocket;
		sockaddr_in a_ClientSocketAddress;
		int a_Length = sizeof(a_ClientSocketAddress);

		a_ClientSocket = ::accept(m_ServerSocket, reinterpret_cast<sockaddr*>(&a_ClientSocketAddress), &a_Length);
		if(a_ClientSocket == INVALID_SOCKET){
			m_doLoop = false;
			return;
		}

		std::cout << "accepted connection from " << inet_ntoa(a_ClientSocketAddress.sin_addr) << ", port = " << ntohs(a_ClientSocketAddress.sin_port) << std::endl;

		// 未接続から接続済みに移動する
		m_NonConnectedSockets.front()->connection(&a_ClientSocket, &a_ClientSocketAddress);
		m_ConnectedSockets.push_back(m_NonConnectedSockets.front());
		m_NonConnectedSockets.pop_front();
	}

	// 接続済みの中から接続が終了したものを探索し、未接続に移動する
	// 接続が終了してくれたCSocketが通知してくれると効率がよい
	std::list<CSocket*>::iterator p = m_ConnectedSockets.begin();
	const std::list<CSocket*>::iterator end = m_ConnectedSockets.end();
	while(true){
		if(p == end){
			break;
		}

		if(!(*p)->isConnect()){
			// 未接続
			m_NonConnectedSockets.push_back(*p);
			p = m_ConnectedSockets.erase(p);
			continue;
		}
		p++;
	}
}
