﻿#include <winsock2.h>
#include <iostream>

#include "CSocket.h"

////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
////////////////////////////////////////////////////////////////////////////////
CSocket::CSocket()
{
	m_Socket = INVALID_SOCKET;
	m_isConnect = false;
	m_Buffer.allocate(100);
}

////////////////////////////////////////////////////////////////////////////////
// デストラクタ
////////////////////////////////////////////////////////////////////////////////
CSocket::~CSocket()
{
	uninitialize();
}

////////////////////////////////////////////////////////////////////////////////
// 初期化
////////////////////////////////////////////////////////////////////////////////
bool CSocket::initialize()
{
	int a_Error = WSAStartup(MAKEWORD(2,0), &m_WsaData);
	if(a_Error != 0){
		switch(a_Error){
		case WSASYSNOTREADY:
			throw "WSASYSNOTREADY";
			break;
		case WSAVERNOTSUPPORTED:
			throw "WSAVERNOTSUPPORTED";
			break;
		case WSAEINPROGRESS:
			throw "WSAEINPROGRESS";
			break;
		case WSAEPROCLIM:
			throw "WSAEPROCLIM";
			break;
		case WSAEFAULT:
			throw "WSAEFAULT";
			break;
		}
		return false;
	}

	/*
	std::cout << "バージョン = " << static_cast<int>(wsaData.wHighVersion) << "." << static_cast<int>(wsaData.wHighVersion >> 8) << "\n"
	          << "description = " << wsaData.szDescription << "\n"
	          << "status = " << wsaData.szSystemStatus << "\n" << std::endl;
	*/

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 非初期化
////////////////////////////////////////////////////////////////////////////////
bool CSocket::uninitialize()
{
	disconnection();
	WSACleanup();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// connection
////////////////////////////////////////////////////////////////////////////////
bool CSocket::connection(char* in_pIpAddress, unsigned short in_PortNumber)
{
	m_Socket = socket(AF_INET, SOCK_STREAM, 0);
	if(m_Socket == INVALID_SOCKET){
		//std::cout << "socket failed: " << WSAGetLastError() << std::endl;
		return false;
	}

	m_SocketAddress.sin_family = AF_INET;
	m_SocketAddress.sin_port = htons(in_PortNumber);
	m_SocketAddress.sin_addr.S_un.S_addr = inet_addr(in_pIpAddress);
	if(connect(m_Socket, reinterpret_cast<sockaddr*>(&m_SocketAddress), sizeof(m_SocketAddress)) == SOCKET_ERROR){
		closesocket(m_Socket);
		m_Socket = INVALID_SOCKET;
		return false;
	}
	m_isConnect = true;

	start(true);
	return true;
}

bool CSocket::connection(SOCKET* io_pSocket, sockaddr_in* io_pSockAddress)
{
	m_Socket = *io_pSocket;
	m_SocketAddress = *io_pSockAddress;
	m_isConnect = true;

	start(true);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// disconnection
////////////////////////////////////////////////////////////////////////////////
bool CSocket::disconnection()
{
	if(closesocket(m_Socket) == SOCKET_ERROR){
		return false;
	}
	m_Socket = INVALID_SOCKET;
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// shutdown
////////////////////////////////////////////////////////////////////////////////
bool CSocket::shutdown(unsigned int in_Kind)
{
	int a_Result = SOCKET_ERROR;
	switch(in_Kind){
	case 0:
		// 送信中止
		a_Result = ::shutdown(m_Socket, SD_SEND);
		break;
	case 1:
		// 受信中止
		a_Result = ::shutdown(m_Socket, SD_RECEIVE);
		break;
	case 2:
		// 送受信中止
		a_Result = ::shutdown(m_Socket, SD_BOTH);
		break;
	default:
		// 無効なコマンド
		throw "UNKNOWN COMMAND.";
		break;
	}
	if(a_Result == SOCKET_ERROR){
		return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// send
////////////////////////////////////////////////////////////////////////////////
bool CSocket::send(const CommunicationBuffer* in_pCommunicationBuffer)
{
	if(::send(m_Socket, in_pCommunicationBuffer->m_Buffer, in_pCommunicationBuffer->m_Length, 0) == SOCKET_ERROR){
		return false;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// receive
////////////////////////////////////////////////////////////////////////////////
bool CSocket::receive(CommunicationBuffer** out_ppCommunicationBuffer)
{
	if(m_Buffer.empty()){
		return false;
	}

	*out_ppCommunicationBuffer = m_Buffer.getReadPointer();
	m_Buffer.nextReadBuffer();
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 受信を行う
////////////////////////////////////////////////////////////////////////////////
void CSocket::run()
{
	if(!m_Buffer.full()){
		m_Buffer.getWritePointer()->m_Length = recv(m_Socket, m_Buffer.getWritePointer()->m_Buffer, MAX_STRING_BUFFER_SIZE, 0);
		if(m_Buffer.getWritePointer()->m_Length == 0){
			std::cout << "disconnection" << std::endl;
			m_isConnect = false;
			m_doLoop = false;
			return;
		}
		else if(m_Buffer.getWritePointer()->m_Length == SOCKET_ERROR){
			std::cout << "socket error" << std::endl;
			m_isConnect = false;
			m_doLoop = false;
			return;
		}

		//std::cout << m_Buffer.getWritePointer()->m_Buffer << std::endl;
		m_Buffer.nextWriteBuffer();
	}
}
