﻿#ifndef INC_CSocket
#define INC_CSocket

// インクルードは必ず必要な物だけ
// 内部で利用するだけの物はソースに記述
// winsock2.hはwindows.hより先にインクルードしなければならないらしい
//#include <winsock2.h>
#include "CThread.h"
#include "CRingBuffer.h"

// 関数名はローワキャメルケースとする

const int MAX_STRING_BUFFER_SIZE = 256;

struct CommunicationBuffer
{
	char m_Buffer[MAX_STRING_BUFFER_SIZE];
	// 格納されている文字数
	size_t m_Length;

	CommunicationBuffer()
	{
		m_Length = 0;
	}
};

////////////////////////////////////////////////////////////////////////////////
// CSocket
////////////////////////////////////////////////////////////////////////////////
class CSocket : public CThread
{
	////////////////////////////////////////////////////////////////////////////////
	// メンバ変数
	////////////////////////////////////////////////////////////////////////////////
protected:
	WSADATA m_WsaData;
	SOCKET m_Socket;
	sockaddr_in m_SocketAddress;

	bool m_isConnect;
	CRingBuffer<CommunicationBuffer> m_Buffer;

	////////////////////////////////////////////////////////////////////////////////
	// コンストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	CSocket();

	////////////////////////////////////////////////////////////////////////////////
	// デストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	virtual ~CSocket();

	////////////////////////////////////////////////////////////////////////////////
	// static関数、コールバック関数
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 操作メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	// 初期化
	virtual bool initialize();
	// 非初期化
	virtual bool uninitialize();
	// connection
	virtual bool connection(char* in_pIpAddress = "127.0.0.1", unsigned short in_PortNumber = 12345);
	virtual bool connection(SOCKET* io_pSocket, sockaddr_in* io_pSockAddress);
	// disconnection
	virtual bool disconnection();
	// shutdown
	virtual bool shutdown(unsigned int in_Kind);
	// send
	virtual bool send(const CommunicationBuffer* in_pCommunicationBuffer);
	// receive
	virtual bool receive(CommunicationBuffer** out_ppCommunicationBuffer);

protected:
	// スレッドループ
	void run();


	////////////////////////////////////////////////////////////////////////////////
	// 属性メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	SOCKET* getSocket()
	{
		return &m_Socket;
	}

	sockaddr* getSocketAddress()
	{
		return reinterpret_cast<sockaddr*>(&m_SocketAddress);
	}

	////////////////////////////////////////////////////////////////////////////////
	// イテレーション
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 状態
	////////////////////////////////////////////////////////////////////////////////
public:
	bool isConnect() const
	{
		return m_isConnect;
	}

	////////////////////////////////////////////////////////////////////////////////
	// 演算子
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 純粋仮想関数、定義のみ（非実装提供）
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 内部クラス
	////////////////////////////////////////////////////////////////////////////////


};

#endif //INC_CSocket
