﻿// 京

#include "CThread.h"

////////////////////////////////////////////////////////////////////////////////
// コンストラクタ
////////////////////////////////////////////////////////////////////////////////
CThread::CThread()
{
	m_hThread = NULL;
	m_doLoop  = false;
	m_isExist = false;
}

////////////////////////////////////////////////////////////////////////////////
// デストラクタ
////////////////////////////////////////////////////////////////////////////////
CThread::~CThread()
{
	CriticalSectionID begin = m_CriticalSectionList.begin();
	CriticalSectionID end   = m_CriticalSectionList.end();
	for(CriticalSectionID p = begin; p != end; p++){
		DeleteCriticalSection(*p);
	}
}

////////////////////////////////////////////////////////////////////////////////
// static関数、コールバック関数
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// スレッドコールバック用
////////////////////////////////////////////////////////////////////////////////
unsigned int __stdcall CThread::procThread(void* in_pParameter)
{
	CThread* pClass = static_cast<CThread*>(in_pParameter);

	while(pClass->m_isExist){
		pClass->run();

		if(!pClass->m_doLoop){
			break;
		}
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// 操作メソッド
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// スレッドを開始する
////////////////////////////////////////////////////////////////////////////////
bool CThread::start(bool in_doLoop, UINT in_Flag)
{
	// デフォルトではすぐに開始する
	// CREATE_SUSPENDEDを指定でサスペンド

	UINT ThreadId;
	m_hThread = reinterpret_cast<HANDLE>(_beginthreadex(0, 0, procThread, this, in_Flag, &ThreadId));
	if(m_hThread == NULL){
		return false;
	}
	m_doLoop = in_doLoop;
	m_isExist = true;

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// スレッドを終了する
////////////////////////////////////////////////////////////////////////////////
void CThread::stop()
{
	if(m_isExist){
		m_isExist = false;

		if(m_hThread != NULL){
			// スレッドの終了を待つ
			::WaitForSingleObject(m_hThread, INFINITE);
			// スレッドハンドルを閉じる
			::CloseHandle(m_hThread);
			m_hThread = NULL;
		}
	}
}

////////////////////////////////////////////////////////////////////////////////
// スレッドを一時停止する
////////////////////////////////////////////////////////////////////////////////
UINT CThread::suspend()
{
	DWORD Status = ::SuspendThread(m_hThread);
	if(Status == 0xFFFFFFFF){
		return ::GetLastError();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// スレッドを再スタートする
////////////////////////////////////////////////////////////////////////////////
UINT CThread::resume()
{
	DWORD Status = ::ResumeThread(m_hThread);
	if(Status == 0xFFFFFFFF){
		return ::GetLastError();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// スレッドループ
////////////////////////////////////////////////////////////////////////////////
void CThread::run()
{
	Sleep(100);
}

////////////////////////////////////////////////////////////////////////////////
// クリティカルセクション制御IDを取得する
////////////////////////////////////////////////////////////////////////////////
const CriticalSectionID CThread::createCriticalSectionID()
{
	MyCriticalSection* temp = new MyCriticalSection;
	InitializeCriticalSection(temp);
	m_CriticalSectionList.push_front(temp);
	return m_CriticalSectionList.begin();
}

////////////////////////////////////////////////////////////////////////////////
// クリティカルセクション制御IDを開放する
////////////////////////////////////////////////////////////////////////////////
void CThread::deleteCriticalSectionID(const CriticalSectionID in_CriticalSectionID)
{
	DeleteCriticalSection(*in_CriticalSectionID);
	m_CriticalSectionList.erase(in_CriticalSectionID);
}

////////////////////////////////////////////////////////////////////////////////
// クリティカルセクションに入る
////////////////////////////////////////////////////////////////////////////////
void CThread::enterCriticalSectionID(const CriticalSectionID in_CriticalSectionID)
{
	EnterCriticalSection(*in_CriticalSectionID);
}

////////////////////////////////////////////////////////////////////////////////
// クリティカルセクションから出る
////////////////////////////////////////////////////////////////////////////////
void CThread::leaveCriticalSectionID(const CriticalSectionID in_CriticalSectionID)
{
	LeaveCriticalSection(*in_CriticalSectionID);
}

////////////////////////////////////////////////////////////////////////////////
// 属性メソッド
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// プロセスプライオリティの設定
////////////////////////////////////////////////////////////////////////////////
UINT CThread::setPriorityClass(DWORD in_PriorityClass)
{
	// 優先順位クラスが最高の時は安全の為、エラーとして返す。
	if(in_PriorityClass == REALTIME_PRIORITY_CLASS){
		return -1;
	}

	BOOL Status = ::SetPriorityClass(m_hThread, in_PriorityClass);
	if(!Status){
		return ::GetLastError();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// スレッドの優先度を設定する
////////////////////////////////////////////////////////////////////////////////
UINT CThread::setPriority(int in_Priority)
{
	BOOL Status = ::SetThreadPriority(m_hThread, in_Priority);
	if(!Status){
		return ::GetLastError();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// スレッドの優先度を優先度を最低( = 1)にする
////////////////////////////////////////////////////////////////////////////////
UINT CThread::setPriorityMinimum()
{
	BOOL Status = ::SetThreadPriority(m_hThread, THREAD_PRIORITY_IDLE);
	if(!Status){
		return ::GetLastError();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// スレッドの優先度を優先度を最大( = 15)にする
////////////////////////////////////////////////////////////////////////////////
UINT CThread::setPriorityMaxisim()
{
	BOOL Status = ::SetThreadPriority(m_hThread, THREAD_PRIORITY_TIME_CRITICAL);
	if(!Status){
		return ::GetLastError();
	}

	return 0;
}

////////////////////////////////////////////////////////////////////////////////
// 状態
////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////
// スレッドが存在するか調べる
////////////////////////////////////////////////////////////////////////////////
bool CThread::isExist()
{
	return m_isExist;
}
