﻿// 京

#ifndef INC_CThread
#define INC_CThread

#include <windows.h>
#include <process.h>
#include <list>

typedef CRITICAL_SECTION MyCriticalSection;
typedef std::list<MyCriticalSection*> CriticalSectionList;
typedef CriticalSectionList::iterator CriticalSectionID;

////////////////////////////////////////////////////////////////////////////////
// CThread
// Windows用スレッドクラス
////////////////////////////////////////////////////////////////////////////////
class CThread
{
	////////////////////////////////////////////////////////////////////////////////
	// メンバ変数
	////////////////////////////////////////////////////////////////////////////////
protected:
	// Windows用スレッド制御用
	HANDLE m_hThread;
	// スレッド関数ループ管理用
	bool m_doLoop;
	// スレッド存在管理用
	bool m_isExist;

private:
	// クリティカルセクション制御用
	CriticalSectionList m_CriticalSectionList;

	////////////////////////////////////////////////////////////////////////////////
	// コンストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	CThread();

	////////////////////////////////////////////////////////////////////////////////
	// デストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	virtual ~CThread();

	////////////////////////////////////////////////////////////////////////////////
	// static関数、コールバック関数
	////////////////////////////////////////////////////////////////////////////////
protected:
	static unsigned int __stdcall procThread(void* in_pParameter);

	////////////////////////////////////////////////////////////////////////////////
	// 操作メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	// スタート(DoLoop: true - 繰り返す, false - 一回のみ, Flag: スレッドの設定)
	virtual bool start(bool in_doLoop = false, UINT in_Flag = 0);
	// ストップ
	virtual void stop();
	// 一時停止
	virtual UINT suspend();
	// スレッド再スタート
	virtual UINT resume();
protected:
	// スレッドループ
	virtual void run();

public:
	// クリティカルセクション制御IDを取得する
	const CriticalSectionID createCriticalSectionID();
	// クリティカルセクション制御IDを開放する
	void deleteCriticalSectionID(const CriticalSectionID in_CriticalSectionID);
	// クリティカルセクションに入る
	void enterCriticalSectionID(const CriticalSectionID in_CriticalSectionID);
	// クリティカルセクションから出る
	void leaveCriticalSectionID(const CriticalSectionID in_CriticalSectionID);


	////////////////////////////////////////////////////////////////////////////////
	// 属性メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	// プロセスプライオリティの設定
	virtual UINT setPriorityClass(DWORD in_PriorityClass);
	// 優先度の設定
	virtual UINT setPriority(int in_Priority);
	// 優先度を最低にする
	virtual UINT setPriorityMinimum();
	// 優先度を最大にする
	virtual UINT setPriorityMaxisim();

	////////////////////////////////////////////////////////////////////////////////
	// イテレーション
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 状態
	////////////////////////////////////////////////////////////////////////////////
public:
	// スレッドが存在するか調べる
	virtual bool isExist();

	////////////////////////////////////////////////////////////////////////////////
	// 演算子
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 純粋仮想関数、定義のみ（非実装提供）
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 内部クラス
	////////////////////////////////////////////////////////////////////////////////
};

#endif //INC_CThread
