﻿// 京
#include "../../common/Environment.h"

#include "CClientModel.h"

#include <iostream>
#include <string>


////////////////////////////////////////////////////////////////////////////////
// constructor
////////////////////////////////////////////////////////////////////////////////
CClientModel::CClientModel()
{
	m_isLogin = false;
	initializeSpeex(&m_SpeexData);
}

////////////////////////////////////////////////////////////////////////////////
// destructor
////////////////////////////////////////////////////////////////////////////////
CClientModel::~CClientModel()
{
	uninitializeSpeex(&m_SpeexData);
}

////////////////////////////////////////////////////////////////////////////////
// initialize
////////////////////////////////////////////////////////////////////////////////
bool CClientModel::initialize()
{
	m_Connection.initialize();

	// 現状buffer_sizeが160の倍数でなければ、圧縮の関係上その間にノイズが入る
	// buffer_size = 22050 * bitrate / 8 * channel * time / 1000;
	// buffer_size が160の倍数

	// 一般的にはバッファは連続しているため、好きな位置で切ることが可能だが、現状の
	// 作りではバッファをリングバッファとして分割しているため、好きな位置で切ることができない。
	// よって、今後はバッファを好きな間隔で切ることが可能な作りにする必要がある。

	m_InputWave.initialize();
	//m_InputWave.setDataFullCallBack(&DataFullCallBackFunction);
	if(!m_InputWave.open()){
		return 0;
	}

	m_OutputWave.initialize();
	if(!m_OutputWave.open()){
		return 0;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// initialize of speex
////////////////////////////////////////////////////////////////////////////////
bool CClientModel::initializeSpeex(SpeexData* io_pSpeexData)
{
	io_pSpeexData->st = speex_encoder_init(speex_lib_get_mode(SPEEX_MODEID_NB));
	io_pSpeexData->dec = speex_decoder_init(speex_lib_get_mode(SPEEX_MODEID_NB));

	/* BEGIN: You probably don't need the following in a real application */
#if 0
	callback.callback_id = SPEEX_INBAND_CHAR;
	callback.func = speex_std_char_handler;
	callback.data = stderr;
	speex_decoder_ctl(dec, SPEEX_SET_HANDLER, &callback);

	callback.callback_id = SPEEX_INBAND_MODE_REQUEST;
	callback.func = speex_std_mode_request_handler;
	callback.data = st;
	speex_decoder_ctl(dec, SPEEX_SET_HANDLER, &callback);
#endif
	/* END of unnecessary stuff */

	spx_int32_t tmp;
	tmp=1;
	speex_decoder_ctl(io_pSpeexData->dec, SPEEX_SET_ENH, &tmp);
	tmp=0;
	speex_encoder_ctl(io_pSpeexData->st, SPEEX_SET_VBR, &tmp);
	tmp=8;
	speex_encoder_ctl(io_pSpeexData->st, SPEEX_SET_QUALITY, &tmp);
	/*
	tmp=1;
	speex_encoder_ctl(st, SPEEX_SET_COMPLEXITY, &tmp);
	*/
	tmp = 8000;
	//speex_encoder_ctl(st, SPEEX_SET_SAMPLING_RATE, &tmp);
	//speex_decoder_ctl(dec, SPEEX_SET_SAMPLING_RATE, &tmp);
	//tmp = 8000;
	//speex_encoder_ctl(st, SPEEX_SET_BITRATE, &tmp);

	spx_int32_t frame_size;
	speex_encoder_ctl(io_pSpeexData->st, SPEEX_GET_FRAME_SIZE, &frame_size);

	/* Turn this off if you want to measure SNR (on by default) */
	/*
	tmp=1;
	speex_encoder_ctl(st, SPEEX_SET_HIGHPASS, &tmp);
	speex_decoder_ctl(dec, SPEEX_SET_HIGHPASS, &tmp);
	*/

	speex_encoder_ctl(io_pSpeexData->st, SPEEX_GET_LOOKAHEAD, &io_pSpeexData->skip_group_delay);
	speex_decoder_ctl(io_pSpeexData->dec, SPEEX_GET_LOOKAHEAD, &tmp);
	io_pSpeexData->skip_group_delay += tmp;

	speex_bits_init(&io_pSpeexData->bits);

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// uninitialize of speex
////////////////////////////////////////////////////////////////////////////////
bool CClientModel::uninitializeSpeex(SpeexData* io_pSpeexData)
{
	speex_encoder_destroy(io_pSpeexData->st);
	speex_decoder_destroy(io_pSpeexData->dec);
	speex_bits_destroy(&io_pSpeexData->bits);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// イベント追加
////////////////////////////////////////////////////////////////////////////////
bool CClientModel::addEvent(std::string in_Event)
{
	m_Events.push_back(in_Event);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 処理
////////////////////////////////////////////////////////////////////////////////
bool CClientModel::process()
{
	// 通信クラスからデータ取得
	CommunicationData* a_pData;
	if(m_Connection.receive(&a_pData)){
		// 座標データの場合
		if(a_pData->m_DataType == 'p'){
			// 同じ名前のアバターが存在するかチェックする
			bool check = false;
			AvatarInformationList::iterator p = m_AvatarInformations.begin();
			while(true){
				// 同じ名前のアバターが存在しない場合、新たに作成する
				if(p == m_AvatarInformations.end()){
					AvatarInformation temp;
					strcpy(temp.m_Name, a_pData->m_Data.m_AvatarData.m_Name);
					m_AvatarInformations.push_front(temp);
					p = m_AvatarInformations.begin();
					break;
				}
				if(strcmp(p->m_Name, a_pData->m_Data.m_AvatarData.m_Name) == 0){
					break;
				}
				p++;
			}
			p->m_Position[0] = a_pData->m_Data.m_AvatarData.m_Position[0];
			p->m_Position[1] = a_pData->m_Data.m_AvatarData.m_Position[1];
			p->m_Position[2] = a_pData->m_Data.m_AvatarData.m_Position[2];
			p->m_Direction[0] = a_pData->m_Data.m_AvatarData.m_Direction[0];
			p->m_Direction[1] = a_pData->m_Data.m_AvatarData.m_Direction[1];
			p->m_Direction[2] = a_pData->m_Data.m_AvatarData.m_Direction[2];
			p->m_Radias = a_pData->m_Data.m_AvatarData.m_Radias;
		}
		// 音声データの場合
		if(a_pData->m_DataType == 'v'){
			static char buffer[3000];
			static int length = 0;
			short out_short[FRAME_SIZE];

			// リセット
			//speex_bits_reset(&m_SpeexData.bits);

			// 圧縮ビットストリームをセット（同時にリセットされる）
			speex_bits_read_from(&m_SpeexData.bits, a_pData->m_Data.m_Voice.m_Stream, a_pData->m_Data.m_Voice.m_Length);

			// 解凍
			speex_decode_int(m_SpeexData.dec, &m_SpeexData.bits, out_short);

			// Convert 16 -> 8 bits
			for(int i = 0; i < FRAME_SIZE; i++){
				buffer[length] = (out_short[i] ^ 0x8000) >> 8;
				length++;
			}

			if(a_pData->m_Data.m_Voice.m_isEnd){
				StructWaveBuffer a_OutputWaveData;
				a_OutputWaveData.m_pWaveBuffer = buffer;
				a_OutputWaveData.m_BufferSize = length;
				a_OutputWaveData.m_RecorMilliSecond = 100;
				m_OutputWave.setWaveDataPointer(&a_OutputWaveData);
				length = 0;
				//std::cout << "reset" << std::endl;
			}
		}
	}

	// イベント処理
	if(!m_Events.empty()){
		std::string& a_rEvent = m_Events.front();
		// connect
		if(a_rEvent == "CONNECT"){
			//m_Connection.connection();
			if(m_Connection.connection(m_IpAddress, m_PortNumber)){
				std::cout << "seccess connection" << std::endl;
			}
			else{
				std::cout << "failure connection" << std::endl;
			}
		}
		// disconnect
		else if(a_rEvent == "DISCONNECT"){
			if(m_Connection.disconnection()){
				std::cout << "sccess disconnection" << std::endl;
			}
			else{
				std::cout << "failure disconnection" << std::endl;
			}
		}
		// ログイン処理
		else if(a_rEvent == "LOGIN"){
			m_Connection.login(m_LoginName.c_str());
			m_isLogin = true;
		}
		// ログアウト
		else if(a_rEvent == "LOGOUT"){
			m_isLogin = false;
			m_Connection.logout();
		}
		// 座標取得
		else if(a_rEvent == "POSITION"){
			m_Connection.requestPosition();
		}
		// アクションコマンド
		// 前方へ移動
		else if(a_rEvent == "FRONT"){
			m_Connection.requestMove('f');
		}
		// 後方へ移動
		else if(a_rEvent == "BACK"){
			m_Connection.requestMove('b');
		}
		// 右旋回
		else if(a_rEvent == "TURN_RIGHT"){
			m_Connection.requestMove('r');
		}
		// 左旋回
		else if(a_rEvent == "TURN_LEFT"){
			m_Connection.requestMove('l');
			m_InputWave.getWaveDataPointer();
		}

		m_Events.pop_front();
	}

	try{
		if(m_isLogin){
			m_InputWave.start();
			m_OutputWave.start();
		}
		else{
			m_InputWave.stop();
			m_OutputWave.stop();
		}
	}
	catch(char* str){
		std::cout << str << std::endl;
	}

	// 音声入力
#if 0
	m_OutputWave.setWaveDataPointer(m_InputWave.getWaveDataPointer());
#else
	StructWaveBuffer* a_pInputWaveData = m_InputWave.getWaveDataPointer();
	if(a_pInputWaveData != NULL){
		char* p1 = a_pInputWaveData->m_pWaveBuffer;

		short in_short[FRAME_SIZE];

		int bitCount = 0;
		while(true){
			// 圧縮
			// Convert 8 -> 16 bits
			for(int i = 0; i < FRAME_SIZE; i++){
				// 量子化ビット数が8ビットの場合は、0〜255 の範囲で振幅値を表現します。128 が中心(振幅なし)です。
				// 16ビットの場合は、-32768〜32767 の範囲で振幅値を表現します。0 が中心(振幅なし)です。
				if(p1 < a_pInputWaveData->m_pWaveBuffer + a_pInputWaveData->m_BufferSize){
					in_short[i] = (*(p1 + i) << 8) ^ 0x8000;
				}
				else{
					in_short[i] = (128 << 8) ^ 0x8000;
				}
			}
			p1 += FRAME_SIZE;

			// リセット
			speex_bits_reset(&m_SpeexData.bits);

			// 圧縮
			speex_encode_int(m_SpeexData.st, in_short, &m_SpeexData.bits);

			// 圧縮ビットストリーム取得
			char cbits[200];
			int nbBytes = speex_bits_write(&m_SpeexData.bits, cbits, 200);
			bitCount += m_SpeexData.bits.nbBits;

			// 送信
			if(p1 >= a_pInputWaveData->m_pWaveBuffer + a_pInputWaveData->m_BufferSize){
				m_Connection.sendVoice(cbits, nbBytes, 1);
				break;
			}
			else{
				m_Connection.sendVoice(cbits, nbBytes, 0);
			}
		}
	}
#endif

	return true;
}

