﻿#ifndef INC_CClientModel
#define INC_CClientModel

// インクルードは必ず必要な物だけ
// 内部で利用するだけの物はソースに記述
#include "../../common/Data.h"
//#include "../../common/CConnection.h"
#include "../../common/CClientConnection.h"
#include "../../common/CInputWave.h"
#include "../../common/COutputWave.h"
#include "../../common/CWaveData.h"

#include <list>

// アバター管理リスト
typedef std::list<AvatarInformation> AvatarInformationList;

#include <speex/speex.h>
#pragma comment(lib, "libspeex.lib")
#pragma comment(lib, "libspeexdsp.lib")
/* BEGIN: You probably don't need the following in a real application */
#include <speex/speex_callbacks.h>
/* END of unnecessary stuff */

struct SpeexData{
	// エンコーダハンドル
	void *st;
	// デコーダハンドル
	void *dec;
	// 不明
	spx_int32_t skip_group_delay;
	// 不明
	SpeexBits bits;
	/* BEGIN: You probably don't need the following in a real application */
	SpeexCallback callback;
	/* END of unnecessary stuff */
};

static const unsigned int FRAME_SIZE = 160;


// 関数名はローワキャメルケースとする

////////////////////////////////////////////////////////////////////////////////
// CClientModel
////////////////////////////////////////////////////////////////////////////////
class CClientModel
{
	////////////////////////////////////////////////////////////////////////////////
	// メンバ変数
	////////////////////////////////////////////////////////////////////////////////
private:
	char m_IpAddress[20];
	unsigned short m_PortNumber;

	// アバター最大ロード数
	static const int MAX_AVATAR_NUMBER = 10;
	// アバター
	AvatarInformationList m_AvatarInformations;
	// 通信クラス
	CClientConnection m_Connection;

	// ログイン情報（正確なログイン情報は定期的にサーバに問い合わせる必要あり）
	bool m_isLogin;

	// 音声クラス
	CInputWave m_InputWave;
	COutputWave m_OutputWave;

	// speex関連
	SpeexData m_SpeexData;

	std::list<std::string> m_Events;

	// ログイン名
	std::string m_LoginName;

	////////////////////////////////////////////////////////////////////////////////
	// コンストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	CClientModel();

	////////////////////////////////////////////////////////////////////////////////
	// デストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	~CClientModel();

	////////////////////////////////////////////////////////////////////////////////
	// static関数、コールバック関数
	////////////////////////////////////////////////////////////////////////////////

		////////////////////////////////////////////////////////////////////////////////
	// 操作メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	// 初期化
	bool initialize();
	// initialize of speex
	bool initializeSpeex(SpeexData* io_pSpeexData);
	// uninitialize of speex
	bool uninitializeSpeex(SpeexData* io_pSpeexData);
	// イベント追加
	bool addEvent(std::string);
	// 処理
	bool process();

	////////////////////////////////////////////////////////////////////////////////
	// 属性メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	void setLoginName(std::string in_LoginName)
	{
		m_LoginName = in_LoginName;
	}

	void setServerAddress(const char* in_pIpAddress, unsigned short in_PortNumber)
	{
		strcpy(m_IpAddress, in_pIpAddress);
		m_PortNumber = in_PortNumber;
	}

	const AvatarInformationList* getAvatarInformationList()
	{
		// 別スレッドにより書き込みと同時に読み込む可能性あり（現在未対応）
		return &m_AvatarInformations;
	}

	////////////////////////////////////////////////////////////////////////////////
	// イテレーション
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 状態
	////////////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////////////
	// 演算子
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 純粋仮想関数、定義のみ（非実装提供）
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 内部クラス
	////////////////////////////////////////////////////////////////////////////////


};

#endif //INC_CClientModel
