﻿#include "CClientUI.h"

#include <iostream>

#include <Windows.h>
#include <cstdlib>
#include <iostream>
#include <cmath>
#include <time.h>
//#include <GL/glut.h>
#include "glut.h"

#pragma comment(lib, "ws2_32.lib")

#ifndef M_PI
#define M_PI 3.14159265357989
#endif // M_PI


// key status
long g_KeyStatus = 0;
const int KEY_UP_BIT    = 1 << 0;
const int KEY_DOWN_BIT  = 1 << 1;
const int KEY_RIGHT_BIT = 1 << 2;
const int KEY_LEFT_BIT  = 1 << 3;

CClientUI* CClientUI::sm_pUI = NULL;

////////////////////////////////////////////////////////////////////////////////
// constructor
////////////////////////////////////////////////////////////////////////////////
CClientUI::CClientUI()
{
	sm_pUI = this;
	m_isEnd = false;

	m_CriticalSectionOfEvent = createCriticalSectionID();
}

////////////////////////////////////////////////////////////////////////////////
// destructor
////////////////////////////////////////////////////////////////////////////////
CClientUI::~CClientUI()
{
	deleteCriticalSectionID(m_CriticalSectionOfEvent);
	uninitialize();
}

////////////////////////////////////////////////////////////////////////////////
// 画面を描画する
////////////////////////////////////////////////////////////////////////////////
void CClientUI::display()
{
	// clear buffer
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	// initialize modelview matrix
	glLoadIdentity();

	// 座標系の定義
	// x-y平面を地面
	// z = 0を標高0

	// set look at matrix
	// 現状は上から視点（上空5m）とする
	gluLookAt(
		0, 0, 5, // 視点の位置
		0, 0, 0,  // 視点の目標地点
		0, 1, 0   // 頭の向き
	);

	GLfloat position[] = {-1.0, 1.0, 1.0, 0.0};
	glLightfv(GL_LIGHT0, GL_POSITION, position);

	// アバター描画
	// 環境設定
	GLfloat a_AmbientDiffuse[] = {1.0, 0.0, 0.0, 1.0};
	GLfloat a_Specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat a_Shininess[] = {20.0};
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, a_AmbientDiffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, a_Specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, a_Shininess);
	// set avatar color to white
	glColor3f(1.0, 1.0, 1.0);


	const AvatarInformationList* a_pAvatarInformations = sm_pUI->m_pModel->getAvatarInformationList();
	for(AvatarInformationList::const_iterator p = a_pAvatarInformations->begin(); p != a_pAvatarInformations->end(); p++){
		glPushMatrix();
		{
			glTranslated(p->m_Position[0], p->m_Position[1], p->m_Position[2]);
			glutSolidSphere(p->m_Radias, 10, 10);
			glBegin(GL_LINES);
			{
				glVertex3d(0.0, 0.0, 0.0);
				glVertex3d(p->m_Direction[0], p->m_Direction[1], p->m_Direction[2]);
			}
			glEnd();
		}
		glPopMatrix();
	}

	glutSwapBuffers();
}

////////////////////////////////////////////////////////////////////////////////
// 初期化する
////////////////////////////////////////////////////////////////////////////////
void CClientUI::init()
{
	// background color is black
	glClearColor(0.0, 0.0, 0.0, 1.0);

	// 環境変数設定
	GLfloat ambient[] = {0.2, 0.2, 0.2, 1.0};
	GLfloat diffuse[] = {0.6, 0.6, 0.6, 1.0};
	GLfloat specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat position[] = {-1.0, 1.0, 1.0, 0.0};

	//glEnable(GL_LIGHTING);
	glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, specular);
	glLightfv(GL_LIGHT0, GL_POSITION, position);
	//glEnable(GL_LIGHT0);

	GLfloat material_ambient_diffuse[] = {1.0, 0.0, 0.0, 1.0};
	GLfloat material_specular[] = {1.0, 1.0, 1.0, 1.0};
	GLfloat material_shininess[] = {20.0};

	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material_ambient_diffuse);
	glMaterialfv(GL_FRONT, GL_SPECULAR, material_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, material_shininess);

	glEnable(GL_DEPTH_TEST);
	glEnable(GL_CULL_FACE);

	//glShadeModel(GL_FLAT);

	srand(static_cast<unsigned int>(time(NULL)));
}

////////////////////////////////////////////////////////////////////////////////
// 一般キー入力
////////////////////////////////////////////////////////////////////////////////
void CClientUI::keyboard(unsigned char in_Key, int in_x, int in_y)
{
	switch(in_Key){
	case 'c':
		// connect
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("CONNECT");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		break;

	case 'd':
		// disconnect
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("DISCONNECT");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		break;

	case 'i':
		// login
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("LOGIN");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		break;

	case 'o':
		// logout
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("LOGOUT");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		break;

	case 'p':
		// 座標取得
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("POSITION");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		break;


	case 'q':
	case 'Q':
	case '\033':  /* '\033' は ESC の ASCII コード */
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("END");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		//exit(0);
	default:
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// down special key
////////////////////////////////////////////////////////////////////////////////
void CClientUI::downSpecialKey(int in_Key, int in_x, int in_y)
{
	switch(in_Key){
	case GLUT_KEY_UP:
		g_KeyStatus |= KEY_UP_BIT;
		break;
	case GLUT_KEY_DOWN:
		g_KeyStatus |= KEY_DOWN_BIT;
		break;
	case GLUT_KEY_RIGHT:
		g_KeyStatus |= KEY_RIGHT_BIT;
		break;
	case GLUT_KEY_LEFT:
		g_KeyStatus |= KEY_LEFT_BIT;
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// up special key
////////////////////////////////////////////////////////////////////////////////
void CClientUI::upSpecialKey(int in_Key, int in_x, int in_y)
{
	switch(in_Key){
	case GLUT_KEY_UP:
		g_KeyStatus &= ~KEY_UP_BIT;
		break;
	case GLUT_KEY_DOWN:
		g_KeyStatus &= ~KEY_DOWN_BIT;
		break;
	case GLUT_KEY_RIGHT:
		g_KeyStatus &= ~KEY_RIGHT_BIT;
		break;
	case GLUT_KEY_LEFT:
		g_KeyStatus &= ~KEY_LEFT_BIT;
		break;
	}
}

////////////////////////////////////////////////////////////////////////////////
// timer
////////////////////////////////////////////////////////////////////////////////
void CClientUI::timer(int value)
{
	// 1秒間に60回呼び出される
	glutTimerFunc(1000 / 60, timer, 0);

	// キー入力の状態によりアバター（i = 0のみ）を動かす
	// 前進
	if((g_KeyStatus & KEY_UP_BIT) && !(g_KeyStatus & KEY_DOWN_BIT)){
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("FRONT");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
	}
	// 後退
	else if(!(g_KeyStatus & KEY_UP_BIT) && (g_KeyStatus & KEY_DOWN_BIT)){
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("BACK");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
	}
	// 右旋回
	if((g_KeyStatus & KEY_RIGHT_BIT) && !(g_KeyStatus & KEY_LEFT_BIT)){
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("TURN_RIGHT");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
	}
	// 左旋回
	else if(!(g_KeyStatus & KEY_RIGHT_BIT) && (g_KeyStatus & KEY_LEFT_BIT)){
		sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
		sm_pUI->m_Events.push_back("TURN_LEFT");
		sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
	}

	sm_pUI->enterCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);
	sm_pUI->m_Events.push_back("POSITION");
	sm_pUI->leaveCriticalSectionID(sm_pUI->m_CriticalSectionOfEvent);


	glutPostRedisplay();
}

////////////////////////////////////////////////////////////////////////////////
// サイズ変更
////////////////////////////////////////////////////////////////////////////////
void CClientUI::reshape(int w, int h)
{
	glViewport(0, 0, (GLsizei) w, (GLsizei) h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	if(w <= h){
		glFrustum(-1.0, 1.0, -1.0 * (GLfloat)h / (GLfloat)w, 1.0 * (GLfloat)h / (GLfloat)w, 1.0, 30.0);
	}
	else{
		glFrustum(-1.0 * (GLfloat)w / (GLfloat)h, 1.0 * (GLfloat)w / (GLfloat)h, -1.0, 1.0, 1.0, 30.0);
	}
	glMatrixMode(GL_MODELVIEW);
}

////////////////////////////////////////////////////////////////////////////////
// start input
////////////////////////////////////////////////////////////////////////////////
bool CClientUI::initialize()
{
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// stop input
////////////////////////////////////////////////////////////////////////////////
bool CClientUI::uninitialize()
{
	m_isEnd = true;
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// thread loop
////////////////////////////////////////////////////////////////////////////////
void CClientUI::run()
{
	glutInit(m_pArgc, m_ppArgv);
	glutInitDisplayMode(GLUT_RGBA | GLUT_DEPTH | GLUT_DOUBLE);
	glutInitWindowSize(512, 512);
	glutCreateWindow("internetcity");
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);
	glutSpecialFunc(downSpecialKey);
	glutSpecialUpFunc(upSpecialKey);
	glutTimerFunc(1000 / 60, timer, 0);
	glutReshapeFunc(reshape);
	init();
	glutMainLoop();
}


