﻿#include "../../common/Environment.h"
#include "CServerModel.h"

#include <iostream>

////////////////////////////////////////////////////////////////////////////////
// constructor
////////////////////////////////////////////////////////////////////////////////
CServerModel::CServerModel()
{
	initializeSpeex(&m_SpeexData);
}

////////////////////////////////////////////////////////////////////////////////
// destructor
////////////////////////////////////////////////////////////////////////////////
CServerModel::~CServerModel()
{
	for(size_t i = 0; i < m_Connections.size(); i++){
		delete m_Connections[i].m_pConnection;
	}
	for(AvatarInformationList::iterator p = m_AvatarInformations.begin(); p != m_AvatarInformations.end(); p++){
		delete *p;
	}
	uninitializeSpeex(&m_SpeexData);
}

////////////////////////////////////////////////////////////////////////////////
// initialize
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::initialize()
{
	m_ServerSocket.initialize();
	m_ServerSocket.openPort();
	std::cout << "PortNumber = " << m_ServerSocket.getPortNumber() << std::endl;

	m_Connections.resize(MAX_CONNECTION_NUMBER);
	for(size_t i = 0; i < m_Connections.size(); i++){
		ConnectionTuple temp;
		temp.m_pConnection = new CServerConnection();
		temp.m_pConnection->initialize();
		m_ServerSocket.accept(temp.m_pConnection);
		temp.m_pAvatarInformation = NULL;
		m_Connections[i] = temp;
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// initialize of speex
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::initializeSpeex(SpeexData* io_pSpeexData)
{
	io_pSpeexData->st = speex_encoder_init(speex_lib_get_mode(SPEEX_MODEID_NB));
	io_pSpeexData->dec = speex_decoder_init(speex_lib_get_mode(SPEEX_MODEID_NB));

	/* BEGIN: You probably don't need the following in a real application */
#if 0
	callback.callback_id = SPEEX_INBAND_CHAR;
	callback.func = speex_std_char_handler;
	callback.data = stderr;
	speex_decoder_ctl(dec, SPEEX_SET_HANDLER, &callback);

	callback.callback_id = SPEEX_INBAND_MODE_REQUEST;
	callback.func = speex_std_mode_request_handler;
	callback.data = st;
	speex_decoder_ctl(dec, SPEEX_SET_HANDLER, &callback);
#endif
	/* END of unnecessary stuff */

	spx_int32_t tmp;
	tmp=1;
	speex_decoder_ctl(io_pSpeexData->dec, SPEEX_SET_ENH, &tmp);
	tmp=0;
	speex_encoder_ctl(io_pSpeexData->st, SPEEX_SET_VBR, &tmp);
	tmp=8;
	speex_encoder_ctl(io_pSpeexData->st, SPEEX_SET_QUALITY, &tmp);
	/*
	tmp=1;
	speex_encoder_ctl(st, SPEEX_SET_COMPLEXITY, &tmp);
	*/
	tmp = 8000;
	//speex_encoder_ctl(st, SPEEX_SET_SAMPLING_RATE, &tmp);
	//speex_decoder_ctl(dec, SPEEX_SET_SAMPLING_RATE, &tmp);
	//tmp = 8000;
	//speex_encoder_ctl(st, SPEEX_SET_BITRATE, &tmp);

	spx_int32_t frame_size;
	speex_encoder_ctl(io_pSpeexData->st, SPEEX_GET_FRAME_SIZE, &frame_size);

	/* Turn this off if you want to measure SNR (on by default) */
	/*
	tmp=1;
	speex_encoder_ctl(st, SPEEX_SET_HIGHPASS, &tmp);
	speex_decoder_ctl(dec, SPEEX_SET_HIGHPASS, &tmp);
	*/

	speex_encoder_ctl(io_pSpeexData->st, SPEEX_GET_LOOKAHEAD, &io_pSpeexData->skip_group_delay);
	speex_decoder_ctl(io_pSpeexData->dec, SPEEX_GET_LOOKAHEAD, &tmp);
	io_pSpeexData->skip_group_delay += tmp;

	speex_bits_init(&io_pSpeexData->bits);

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// uninitialize of speex
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::uninitializeSpeex(SpeexData* io_pSpeexData)
{
	speex_encoder_destroy(io_pSpeexData->st);
	speex_decoder_destroy(io_pSpeexData->dec);
	speex_bits_destroy(&io_pSpeexData->bits);
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 処理
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::process()
{
	// 通信クラスからデータ取得
	for(ConnectionVector::iterator p = m_Connections.begin(); p != m_Connections.end(); p++){
		CommunicationData* a_pData;
		if(p->m_pConnection->receive(&a_pData)){
			// ログイン処理（セッションとログインネームと（パスワード）を渡す）
			if(a_pData->m_DataType == 'i'){
				if(login(p, a_pData->m_Data.m_LoginData.m_Name)){
					std::cout << "login " << a_pData->m_Data.m_LoginData.m_Name << std::endl;
				}
			}
			// ログアウト処理
			else if(a_pData->m_DataType == 'o'){
				logout(p, p->m_pAvatarInformation->m_Name);
			}
			// 座標取得
			else if(a_pData->m_DataType == 'p'){
				requestPosition(p);
			}
			// アクションコマンドを処理
			else if(a_pData->m_DataType == 'a'){
				requestMove(p, a_pData->m_Data.m_ActionCommand.m_CommandType);
			}
			// 音声データを格納する
			else if(a_pData->m_DataType == 'v' && p->m_pAvatarInformation != NULL){
				std::cout << "受信" << std::endl;
				short out_short[FRAME_SIZE];

				// リセット
				//speex_bits_reset(&m_SpeexData.bits);

				// 圧縮ビットストリームをセット（同時にリセットされる）
				speex_bits_read_from(&m_SpeexData.bits, a_pData->m_Data.m_Voice.m_Stream, a_pData->m_Data.m_Voice.m_Length);

				// 解凍
				speex_decode_int(m_SpeexData.dec, &m_SpeexData.bits, out_short);

				// Convert 16 -> 8 bits
				for(int i = 0; i < FRAME_SIZE; i++){
					p->m_pAvatarInformation->m_Stream[p->m_pAvatarInformation->m_Length] = (out_short[i] ^ 0x8000) >> 8;
					p->m_pAvatarInformation->m_Length++;
				}

				if(a_pData->m_Data.m_Voice.m_isEnd){
					StructWaveBuffer* a_pOutputWaveData = p->m_pAvatarInformation->m_Voice.getWritePointer();
					a_pOutputWaveData->m_pWaveBuffer = p->m_pAvatarInformation->m_Stream;
					a_pOutputWaveData->m_BufferSize = p->m_pAvatarInformation->m_Length;
					a_pOutputWaveData->m_RecorMilliSecond = 100;
					p->m_pAvatarInformation->m_Length = 0;
					p->m_pAvatarInformation->m_Voice.nextWriteBuffer();
					//std::cout << "reset" << std::endl;
				}
			}
		}
	}
	// 音声データがあれば送信する
	sendVoice();

	// 音声を合成
	// memo: 同時に受信した音声を、同時刻として合成する
	// memo: 圧縮音声データは分割して送信されるため、しばらくためておく必要がある
	// memo: ためた場合、最初が少し遅れるだけで、音がとぎれることはない
	// memo: ただし、自分自信の声は遅れて聞こえるので、自分自身の音は送り返さない
	// memo: また、音声の合成は各キャラクター（クライアント）ごとに生成しなければならない

	// 取得コマンドに対してデータ送信


	// to do
	// 音声を合成する場合、同じ音を何度も合成しないようにするため、
	// 録音時刻や受信時刻を利用して、順番を間違えないようしなければならない

	// 音情報を簡単に扱うため、通信クラスをもう少しまとめなければならない

	// Modelクラスからは通信を行っていることを意識しないようにしたい

	return true;
}

////////////////////////////////////////////////////////////////////////////////
// login
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::login(ConnectionVector::iterator io_ConnectionIterator, const char* in_pLoginName)
{
	AvatarInformation* temp = new AvatarInformation;
	strcpy(temp->m_Name, in_pLoginName);
	temp->m_Position[0] = 0;
	temp->m_Position[1] = 0;
	temp->m_Position[2] = 0;
	temp->m_Direction[0] = 1;
	temp->m_Direction[1] = 0;
	temp->m_Direction[2] = 0;
	temp->m_Radias = 0.3;
	// あとで修正必須（マジックナンバー）
	temp->m_Voice.allocate(10, 100, 2080);
	temp->m_Length = 0;
	m_AvatarInformations.push_back(temp);

	/*
	// ログイン完了送信
	CommunicationData a_SendData;
	a_SendData.m_DataType = 'i';
	strcpy(a_SendData.m_Data.m_LoginData.m_Name, in_pLoginName);
	io_ConnectionIterator->m_pConnection->send(&a_SendData);
	io_ConnectionIterator->m_pAvatarInformation = temp;
	*/
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// logout
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::logout(ConnectionVector::iterator io_ConnectionIterator, const char* in_pLoginName)
{
	// 未ログイン
	if(io_ConnectionIterator->m_pAvatarInformation == NULL){
		return false;
	}

	CommunicationData a_SendData;
	a_SendData.m_DataType = 'o';
	strcpy(a_SendData.m_Data.m_LoginData.m_Name, in_pLoginName);
	io_ConnectionIterator->m_pConnection->send(&a_SendData);

	std::cout << "logout " << io_ConnectionIterator->m_pAvatarInformation->m_Name << std::endl;

	// アバターを削除する
	for(AvatarInformationList::iterator p = m_AvatarInformations.begin(); p != m_AvatarInformations.end(); p++){
		if(*p == io_ConnectionIterator->m_pAvatarInformation){
			delete *p;
		}
		m_AvatarInformations.erase(p);
		break;
	}
	io_ConnectionIterator->m_pAvatarInformation = NULL;
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 送信処理
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::sendData()
{
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// position
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::requestPosition(ConnectionVector::iterator io_ConnectionIterator)
{
	// 未ログイン
	if(io_ConnectionIterator->m_pAvatarInformation == NULL){
		return false;
	}

	if(!m_AvatarInformations.empty()){
		for(AvatarInformationList::iterator p = m_AvatarInformations.begin(); p != m_AvatarInformations.end(); p++){
			CommunicationData a_SendData;
			a_SendData.m_DataType = 'p';
			strcpy(a_SendData.m_Data.m_AvatarData.m_Name, (*p)->m_Name);
			a_SendData.m_Data.m_AvatarData.m_Position[0] = (*p)->m_Position[0];
			a_SendData.m_Data.m_AvatarData.m_Position[1] = (*p)->m_Position[1];
			a_SendData.m_Data.m_AvatarData.m_Position[2] = (*p)->m_Position[2];
			a_SendData.m_Data.m_AvatarData.m_Direction[0] = (*p)->m_Direction[0];
			a_SendData.m_Data.m_AvatarData.m_Direction[1] = (*p)->m_Direction[1];
			a_SendData.m_Data.m_AvatarData.m_Direction[2] = (*p)->m_Direction[2];
			a_SendData.m_Data.m_AvatarData.m_Radias = (*p)->m_Radias;

			if(!io_ConnectionIterator->m_pConnection->send(&a_SendData)){
				std::cout << "do not send position" << std::endl;
			}
		}
	}
	/*
	else{
		CommunicationData a_SendData;
		a_SendBuffer.m_Length = sprintf(a_SendBuffer.m_Buffer, "p;");
		if(!io_pConnection->send(&a_SendBuffer)){
			std::cout << "do not send position" << std::endl;
		}
	}
	*/
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// move
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::requestMove(ConnectionVector::iterator io_ConnectionIterator, char in_pMoveDirection)
{
	const double M_PI = 3.14159265357989;
	// 人の歩く速度を1秒間に1mとする
	const double a_VelocityParTimes = 1.0 / 60;
	// 2秒で振り返る（180度 = pi）とする。
	const double a_RotateParTimes = M_PI / (2 * 60);

	// 未ログイン
	if(io_ConnectionIterator->m_pAvatarInformation == NULL){
		return false;
	}

	AvatarInformation* a_pAvatarInformation = io_ConnectionIterator->m_pAvatarInformation;

	if(in_pMoveDirection == 'f'){
		a_pAvatarInformation->m_Position[0] += a_pAvatarInformation->m_Direction[0] * a_VelocityParTimes;
		a_pAvatarInformation->m_Position[1] += a_pAvatarInformation->m_Direction[1] * a_VelocityParTimes;
		a_pAvatarInformation->m_Position[2] += a_pAvatarInformation->m_Direction[2] * a_VelocityParTimes;
	}
	// 後方へ移動
	else if(in_pMoveDirection == 'b'){
		a_pAvatarInformation->m_Position[0] -= a_pAvatarInformation->m_Direction[0] * a_VelocityParTimes;
		a_pAvatarInformation->m_Position[1] -= a_pAvatarInformation->m_Direction[1] * a_VelocityParTimes;
		a_pAvatarInformation->m_Position[2] -= a_pAvatarInformation->m_Direction[2] * a_VelocityParTimes;
	}
	// 右旋回
	else if(in_pMoveDirection == 'r'){
		double x = a_pAvatarInformation->m_Direction[0];
		double y = a_pAvatarInformation->m_Direction[1];
		a_pAvatarInformation->m_Direction[0] =  x * cos(a_RotateParTimes) + y * sin(a_RotateParTimes);
		a_pAvatarInformation->m_Direction[1] = -x * sin(a_RotateParTimes) + y * cos(a_RotateParTimes);
	}
	// 左旋回
	else if(in_pMoveDirection == 'l'){
		double x = a_pAvatarInformation->m_Direction[0];
		double y = a_pAvatarInformation->m_Direction[1];
		a_pAvatarInformation->m_Direction[0] =  x * cos(a_RotateParTimes) - y * sin(a_RotateParTimes);
		a_pAvatarInformation->m_Direction[1] =  x * sin(a_RotateParTimes) + y * cos(a_RotateParTimes);
	}
	return true;
}

////////////////////////////////////////////////////////////////////////////////
// 音声送信
////////////////////////////////////////////////////////////////////////////////
bool CServerModel::sendVoice()
{
	bool check = false;
	for(ConnectionVector::iterator p = m_Connections.begin(); p != m_Connections.end(); p++){
		if(p->m_pAvatarInformation != NULL){
			check = true;
			StructWaveBuffer* a_pWaveBuffer = p->m_pAvatarInformation->m_Voice.getReadPointer();
			if(a_pWaveBuffer == NULL){
				// 一つでも音声データが存在しなければ送信しない
				// 本来であれば時間で管理すべき
				return false;
			}
		}
	}
	if(!check){
		return false;
	}
	// すべてのコネクションに送信する
	for(ConnectionVector::iterator p = m_Connections.begin(); p != m_Connections.end(); p++){
		if(p->m_pAvatarInformation == NULL){
			continue;
		}
		// すべての音声データを集める
		unsigned char buffer[3000];
		int length = 0;
		for(int i = 0; i < 3000; i++){
			buffer[i] = 128;
		}
		for(ConnectionVector::iterator q = m_Connections.begin(); q != m_Connections.end(); q++){
			if(q->m_pAvatarInformation == NULL){
				continue;
			}
			// 自分自身は合成しない（現在デバッグ中なので合成している）
			StructWaveBuffer* a_pInputWaveData = q->m_pAvatarInformation->m_Voice.getReadPointer();
			for(unsigned int i = 0; i < a_pInputWaveData->m_BufferSize; i++){
				// 量子化ビット数が8ビットの場合は、0〜255 の範囲で振幅値を表現します。128 が中心(振幅なし)です。
				// 16ビットの場合は、-32768〜32767 の範囲で振幅値を表現します。0 が中心(振幅なし)です。
				buffer[i] = static_cast<unsigned char>(buffer[i]) + static_cast<unsigned char>(a_pInputWaveData->m_pWaveBuffer[i]) - 128;
				length = a_pInputWaveData->m_BufferSize;
			}
		}
		unsigned char* p1 = buffer;
		while(true){
			short in_short[FRAME_SIZE];
			int bitCount = 0;
			// Convert 8 -> 16 bits
			for(int i = 0; i < FRAME_SIZE; i++){
				// 量子化ビット数が8ビットの場合は、0〜255 の範囲で振幅値を表現します。128 が中心(振幅なし)です。
				// 16ビットの場合は、-32768〜32767 の範囲で振幅値を表現します。0 が中心(振幅なし)です。
				if(p1 < buffer + length){
					in_short[i] = (*(p1 + i) << 8) ^ 0x8000;
				}
				else{
					in_short[i] = (128 << 8) ^ 0x8000;
				}
			}
			p1 += FRAME_SIZE;

			// リセット
			speex_bits_reset(&m_SpeexData.bits);

			// 圧縮
			speex_encode_int(m_SpeexData.st, in_short, &m_SpeexData.bits);

			// 圧縮ビットストリーム取得
			char cbits[200];
			int nbBytes = speex_bits_write(&m_SpeexData.bits, cbits, 200);
			bitCount += m_SpeexData.bits.nbBits;

			// 送信
			if(p1 >= buffer + length){
				p->m_pConnection->sendVoice(cbits, nbBytes, 1);
				break;
			}
			else{
				p->m_pConnection->sendVoice(cbits, nbBytes, 0);
			}
			std::cout << "送信" << std::endl;
		}
	}
	for(ConnectionVector::iterator p = m_Connections.begin(); p != m_Connections.end(); p++){
		if(p->m_pAvatarInformation == NULL){
			continue;
		}
		p->m_pAvatarInformation->m_Voice.nextReadBuffer();
	}
	return true;
}
