﻿#ifndef INC_CServerModel
#define INC_CServerModel

// インクルードは必ず必要な物だけ
// 内部で利用するだけの物はソースに記述
#include "../../common/Data.h"
#include "../../common/CServerConnection.h"
#include "../../common/CServerSocket.h"
#include <list>
#include <vector>

// 通信クラスリスト
struct ConnectionTuple{
	CServerConnection* m_pConnection;
	AvatarInformation* m_pAvatarInformation;
};
typedef std::vector<ConnectionTuple> ConnectionVector;

// アバター管理リスト
typedef std::list<AvatarInformation*> AvatarInformationList;


#include <speex/speex.h>
#pragma comment(lib, "libspeex.lib")
#pragma comment(lib, "libspeexdsp.lib")
/* BEGIN: You probably don't need the following in a real application */
#include <speex/speex_callbacks.h>
/* END of unnecessary stuff */

struct SpeexData{
	// エンコーダハンドル
	void *st;
	// デコーダハンドル
	void *dec;
	// 不明
	spx_int32_t skip_group_delay;
	// 不明
	SpeexBits bits;
	// BEGIN: You probably don't need the following in a real application
	SpeexCallback callback;
	// END of unnecessary stuff
};

static const unsigned int FRAME_SIZE = 160;


// 関数名はローワキャメルケースとする


////////////////////////////////////////////////////////////////////////////////
// CServerModel
////////////////////////////////////////////////////////////////////////////////
class CServerModel
{
	////////////////////////////////////////////////////////////////////////////////
	// メンバ変数
	////////////////////////////////////////////////////////////////////////////////
private:
	// アバター最大ロード数
	static const int MAX_AVATAR_NUMBER = 10;
	// 最大接続数
	static const int MAX_CONNECTION_NUMBER = 10;
	// アバター
	AvatarInformationList m_AvatarInformations;
	// 通信クラス
	ConnectionVector m_Connections;
	// サーバーソケット
	CServerSocket m_ServerSocket;

	SpeexData m_SpeexData;

	////////////////////////////////////////////////////////////////////////////////
	// コンストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	CServerModel();

	////////////////////////////////////////////////////////////////////////////////
	// デストラクタ
	////////////////////////////////////////////////////////////////////////////////
public:
	~CServerModel();

	////////////////////////////////////////////////////////////////////////////////
	// static関数、コールバック関数
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 操作メソッド
	////////////////////////////////////////////////////////////////////////////////
public:
	// 初期化
	bool initialize();
	// initialize of speex
	bool initializeSpeex(SpeexData* io_pSpeexData);
	// uninitialize of speex
	bool uninitializeSpeex(SpeexData* io_pSpeexData);
	// 処理
	bool process();

private:
	// 各種処理
	// login
	bool login(ConnectionVector::iterator io_ConnectionIterator, const char* in_pLoginName);
	// logout
	bool logout(ConnectionVector::iterator io_ConnectionIterator, const char* in_pLoginName);

	// 送信処理
	bool sendData();

	// position
	bool requestPosition(ConnectionVector::iterator io_ConnectionIterator);
	// move
	bool requestMove(ConnectionVector::iterator io_ConnectionIterator, char in_pMoveDirection);
	// 音声送信
	bool sendVoice();

	////////////////////////////////////////////////////////////////////////////////
	// 属性メソッド
	////////////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////////////
	// イテレーション
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 状態
	////////////////////////////////////////////////////////////////////////////////

	////////////////////////////////////////////////////////////////////////////////
	// 演算子
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 純粋仮想関数、定義のみ（非実装提供）
	////////////////////////////////////////////////////////////////////////////////
	////////////////////////////////////////////////////////////////////////////////
	// 内部クラス
	////////////////////////////////////////////////////////////////////////////////


};

#endif //INC_CServerModel
